use async_trait::async_trait;

use crate::{
    message_handler::user_commands::UResult,
    models::effect::{EffectSet, EffectType, SimpleEffect},
    regular_tasks::queued_command::QueuedCommandContext,
    static_content::room::{Room, RoomEnterTrigger, RoomExitTrigger},
};

use super::effect::{cancel_effect, run_effects};

pub struct RoomEffectEntryTrigger {
    pub effects: Vec<SimpleEffect>,
}

pub struct RoomEffectExitTrigger;

#[async_trait]
impl RoomEnterTrigger for RoomEffectEntryTrigger {
    async fn handle_enter(
        self: &Self,
        ctx: &mut QueuedCommandContext,
        _room: &Room,
    ) -> UResult<()> {
        run_effects(
            &ctx.trans,
            &EffectSet {
                effect_type: EffectType::CurrentRoom,
                effects: self.effects.iter().map(|x| x.into()).collect(),
            },
            &mut ctx.item,
            &Default::default(),
            None,
            0.0,
        )
        .await?;
        Ok(())
    }
}

#[async_trait]
impl RoomExitTrigger for RoomEffectExitTrigger {
    async fn handle_exit(self: &Self, ctx: &mut QueuedCommandContext, _room: &Room) -> UResult<()> {
        for effect in &ctx.item.active_effects {
            if effect.0 == EffectType::CurrentRoom {
                cancel_effect(ctx.trans, &ctx.item, effect).await?;
            }
        }

        Ok(())
    }
}
